import scipy.stats as scst
import numpy
import numpy as np
import math
from eigen4me import eigen_sym

class InitPCA:
    """
                   Calculation of the Principal Components
    """
    def __init__(self, data, f_out):
        self.data = data
        self.f_out = f_out


    def calc_pca(self):   
        pos = 0
        #   pos = mm.check_slash(self.data, pos)
        while True:
            if len(self.data[pos]) > 0:
                if self.data[pos][0] == '/':
                    break
            pos += 1
        pos = pos + 1
        #
        #       n_vars: the number of variables
        #
        n_vars = int(self.data[pos].strip())
        pos += 1
        #
        #       var_names: List of the variable names
        #
        var_names = self.data[pos].split();

        n_data = 0;         #  n_data: the number of cases of the data,
                            #           which will be counted in the following steps
        #
        #       X: The input data matrix, which will be constructed
        #                                   in the following steps from the object data
        #
        X = [0]*n_vars              #  the number of columns = n_vars
        for a in range(0, n_vars):
            X[a] = []
        CaseID = []                 #   The list for the case id variable

        pos += 1
        temp_v = self.data[pos].split()
        #
        #       Add the first case to X
        #
        CaseID.append(temp_v[0])
        temp_p = 0
        for a in range(0, n_vars):  
            X[a].append(float(temp_v[a + 1]))
            temp_p = temp_p + 1
        n_data = n_data + 1
        #
        #       Repeat addition of cases until the line with the slash at the head is met
        #
        while True:
            temp_v = self.data[pos + n_data].split()
            if temp_v[0] == "/":
                break
            CaseID.append(temp_v[0])
            temp_p = 0
            for a in range(0, n_vars):
                X[a].append(float(temp_v[a + 1]))
                temp_p = temp_p + 1
            n_data = n_data + 1

        self.f_out.write("\n")
        self.f_out.write("n_vars = %d\n"%n_vars)
        self.f_out.write("Var_names...\n")
        for vn in var_names:
            self.f_out.write("%10s"%vn)
        self.f_out.write("\n")
        self.f_out.write("\n")
        for i in range(0, n_data):
            self.f_out.write("%10s"%CaseID[i])
            for j in range(0, n_vars):
                self.f_out.write("%10.3f"%X[j][i])
            self.f_out.write("\n")

        Z0 = []
        for j in range(n_vars):
            Z0.append(scst.zscore(X[j]))

        self.f_out.write("\n\nStandardized data...\n\n")
        for j in range(0, n_vars + 1):
            self.f_out.write('{0: >10s}'.format(var_names[j]))  
        self.f_out.write("\n")
        for i in range(0, n_data):
            self.f_out.write('{0: >10s}'.format(CaseID[i]))        
            for j in range(0, n_vars):
                self.f_out.write('{0:10.5f}'.format(Z0[j][i]))    
            self.f_out.write("\n")

        Z = np.array(Z0).transpose()
        #
        #       R = Correlation Matrix
        #

        R = (Z.transpose() @ Z) / n_data

        Lmbd2, V = eigen_sym(R)
        
        Lmbd = []
        for i in range(0, Lmbd2.shape[0]):  
            if Lmbd2[i] > 1.0e-12:          
                Lmbd.append(math.sqrt(Lmbd2[i]))
            else:
                Lmbd.append(0.0)

        n_eign = len(Lmbd)

        self.f_out.write("\nLambda...\n")
        sum_L2 = 0.0
        for i in range(0, n_eign):
            sum_L2 += Lmbd2[i]  
        cum_sum = 0.0
        self.f_out.write("%30s"%"Lambd ")
        self.f_out.write("%10s"%"cum.sqr")
        self.f_out.write("%10s\n"%"%")
        print("\n")
        print(("%30s"%"Lambda") + ("%10s"%"cum.sqr") + ("%10s"%"%") + "\n")

        for i in range(0, n_eign):
            self.f_out.write('{0: >20s}'.format("Lambda-{0}".format(i + 1)))   
            self.f_out.write('{0:>10.5f}'.format(Lmbd[i]))    
            cum_sum += Lmbd2[i]                               
            self.f_out.write("%10.5f"%cum_sum)
            self.f_out.write("%10.2f\n"%(100.0 * cum_sum / sum_L2))
            print('{0: >20s}'.format("Lambda-{0}".format(i + 1)), end = '') 
            print('{0:>10.5f}'.format(Lmbd[i]), end = '')   
            print("%10.5f"%cum_sum, end = "")
            print("%10.2f"%(100.0 * cum_sum / sum_L2))


        s0 = input("\nNumber of components = ")
        n_comp = int(s0)
        if (n_comp > n_vars):
            n_comp = n_vars
        elif (n_comp < 1):
            n_comp = 1    
        if (n_comp > n_eign):   
            n_comp = n_eign
 
        L1 = np.full((n_comp, n_comp), 0.0)
        for i in range(n_comp):
            L1[i, i] = Lmbd[i]

        V1 = V[0: n_comp].transpose()
        
        Pat = V1 @ L1
        
        self.f_out.write("\n\nPattern matrix for the principal components =\n")
        self.f_out.write("%10s"%" ")
        for i in range(0, n_comp):
            self.f_out.write("%10s"%("comp.%d"%(i + 1)))
        self.f_out.write("\n")
        for i in range(0, n_vars):
            self.f_out.write("%10s"%var_names[i + 1])
            for j in range(0, n_comp):
                self.f_out.write("%10.5f"%Pat[i, j])
            self.f_out.write("\n")

        PComp = Z @ V1 @ np.linalg.inv(L1)
            
        return var_names, n_data, n_vars, CaseID, n_comp, Pat, PComp

