import pandas as pd
import numpy as np
import matplotlib.pyplot as plt
import scipy.stats as ss

inflnm = input('Input Data File (*.xlsx) = ')
xlsx = pd.ExcelFile(inflnm)             #   入力ファイル名の設定
data = pd.read_excel(xlsx)              #   読み込み

CaseLabel = data.columns[0]             #   第０列目の変数名
CaseID = data[CaseLabel].values         #   CaseLabelの表す名前の列のデータを取り出してCaseIDで表す
print(CaseID)
YLabel = data.columns[1]                #   第１列目の変数名
Y = data[YLabel].values                 #   YLabelの表す名前の列のデータを取り出してYで表す
print(Y)
XLabel = data.columns[2]                #   第２列目の変数名
X = data[XLabel].values                 #   XL+abelの表す名前の列のデータを取り出してXで表す
print(X)

#
#           (X, Y)の散布図
#
plt.plot(X, Y, 'bo')
plt.xlabel(XLabel)
plt.ylabel(YLabel)
plt.show()
#
#       b0とb1の計算
#
X_mean = X.mean()
Y_mean = Y.mean()
num = ((X - X_mean) * (Y - Y_mean)).sum()
den = ((X - X_mean) ** 2).sum()
b1 = num / den
b0 = Y_mean - b1 * X_mean
print('\nb0 = ', b0)
print('b1 = ', b1)
#
#       回帰直線と散布図の描画
#
plt.plot(X + np.random.uniform(low=-0.15, high=0.15, size = len(X)), Y, 'bo')
plt.plot([X.min(), X.max()], [b0 + b1 * X.min(), b0 + b1 * X.max()], 'g')
plt.title('b0 = {0:.3},  b1 = {1:.3}'.format(b0, b1), fontsize = 16)
plt.xlabel(XLabel, fontsize = 14)
plt.ylabel(YLabel, fontsize = 14)
plt.show()
#
#       決定係数R^2の計算
#
SSReg = (((b0 + b1 * X) - Y_mean) ** 2).sum()
SSTol = ((Y - Y_mean) ** 2).sum()
R2 = SSReg / SSTol
print('\nR2 = ', R2)
#
#       t値の計算
#
n = len(Y)
SSE = ((Y - (b0 + b1 * X)) ** 2).sum()
MSE = SSE / (n - 2)
SSX = ((X - X_mean) ** 2).sum()
s2_b1 = MSE / SSX
s_b1 = s2_b1 ** 0.5
t = b1 / s_b1
pvalue = (1 - ss.t.cdf(t, n - 2)) * 2       #   両側検定
print('t = ', t, '  p = ', pvalue)
#
#       95%信頼区間
#
t_975 = ss.t.ppf(0.975, n - 2)
print('\n95% CI = [{0:<.5}, {1:<.5}]'.format(b1 - t_975 * s_b1, b1 + t_975 * s_b1))
