import numpy as np
import scipy as sc

def common_q(K, vname, X, fout):
    fout.write('\n\nQuantification of {}'.format(vname))
    nq = K - 1
    n = len(X)
    G = np.zeros((n, K))
    for i in range(n):
        G[i, X[i] - 1] = 1.0
    f = G.sum(axis = 0)
    for k in range(K):
        if f[k] == 0:
            print('Category-{0} of variable {1} is not used.'.format(k+1, vname))
            fout.write('\n\nCategory-{0} of variable {1} is not used.'.format(k+1, vname))
            fout.close()
            raise Exception('Category Error !')
    One_n = np.ones((n, 1))
    f = np.reshape(f, (1, K))
    H = G - One_n @ f / n

    HpH = H.T @ H

    wt, vctr = np.linalg.eig(HpH)
    ck_idx = np.abs(wt) > 1.0e-7
    wt = wt[ck_idx]
    vctr = vctr[:, ck_idx]
    idx = np.argsort(-wt)
    wt = wt[idx]
    vctr = vctr.T[idx].T
    fout.write('\neigen values = {}'.format(wt))

    nq = len(wt)   
    qnames = []
    qvars = []
    for h in range(nq): #K - 1):
        qnames.append('q_' + vname + '_{}'.format(h))
        z = H @ np.reshape(vctr[:,h], (K, 1)) / ((wt[h]/n) ** 0.5)
        qvars.append(z)
        fout.write('\nCreated...{0:s}   for lambda = {1:.5}\n'.format(qnames[-1], wt[h]))
        fout.write('                           with omega = ')
        for j in range(K):
            fout.write('{0:.5}   '.format(vctr[j][h]))
            
    return nq, qnames, qvars
